<?php

namespace Drupal\commerce_license_access_control\Plugin\Commerce\LicenseType;

use Drupal\node\Entity\Node;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\user\UserInterface;
use Drupal\commerce_license\Entity\LicenseInterface;
use Drupal\commerce_license\Plugin\Commerce\LicenseType\LicenseTypeBase;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\entity\BundleFieldDefinition;

/**
 * Provides a license type which grants a user access to specific entities.
 *
 * @CommerceLicenseType(
 *  id = "commerce_license_access_control",
 *  label = @Translation("Access Control"),
 *  activation_order_state = "complete",
 * )
 */
class AccessControl extends LicenseTypeBase implements ContainerFactoryPluginInterface {

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition
    );
  }

  /**
   * {@inheritdoc}
   */
  public function buildLabel(LicenseInterface $license) {
    $args = [
      '@entity' => $license->licensed_entity->entity->label(),
    ];
    return $this->t('@entity Acces Control license', $args);
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return [
      'acl_id', '',
      'licensed_entity', '',
    ] + parent::defaultConfiguration();
  }

  /**
   * Get the node licensed by an ACL id.
   *
   * @param int $acl_id
   *   The id of the ACL.
   *
   * @return \Drupal\node\Entity\Node
   *   The licensed node.
   */
  public function getLicensedNode($acl_id) {
    $nid = \Drupal::database()->query("SELECT nid FROM {acl_node} WHERE acl_id = :acl_id", [
      'acl_id' => $acl_id,
    ])->fetchField();
    return Node::load($nid);
  }

  /**
   * {@inheritdoc}
   */
  public function grantLicense(LicenseInterface $license) {
    // Get the owner of the license and add them to the acl.
    $owner = $license->getOwner();
    acl_add_user($license->acl_id->value, $owner->id());

    $node = $this->getLicensedNode($license->acl_id->value);
    if (!empty($node)) {
      \Drupal::entityTypeManager()
        ->getAccessControlHandler('node')
        ->writeGrants($node);
    }
  }

  /**
   * {@inheritdoc}
   */
  public function revokeLicense(LicenseInterface $license) {
    // Get the owner of the license and remove them from the acl.
    $owner = $license->getOwner();
    acl_remove_user($license->acl_id->value, $owner->id());

    $node = $this->getLicensedNode($license->acl_id->value);
    if (!empty($node)) {
      \Drupal::entityTypeManager()
        ->getAccessControlHandler('node')
        ->writeGrants($node);
    }
  }

  /**
   * {@inheritdoc}
   */
  public function checkUserHasExistingRights(UserInterface $user) {
    // Check if user is already added to the acl.
    return ExistingRightsResult::rightsExistIf(
      acl_has_user($this->configuration['acl_id'], $user->id()),
      $this->t("You already have access."),
      $this->t("User @user already has access to ACL @acl.", [
        '@user' => $user->getDisplayName(),
        '@acl' => $this->configuration['acl_id'],
      ])
    );
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $licensed_entity = Node::load($this->configuration['licensed_entity']);
    $form['licensed_entity'] = [
      '#title' => $this->t('Entity'),
      '#description' => $this->t('Select the node to grant access on'),
      '#type' => 'entity_autocomplete',
      '#default_value' => $licensed_entity,
      '#target_type' => 'node',
    ];

    $form['grant_view'] = [
      '#title' => $this->t('Grant view'),
      '#type' => 'checkbox',
      '#default_value' => $this->configuration['grant_view'],
    ];

    $form['grant_update'] = [
      '#title' => $this->t('Grant update'),
      '#type' => 'checkbox',
      '#default_value' => $this->configuration['grant_update'],
    ];

    $form['grant_delete'] = [
      '#title' => $this->t('Grant delete'),
      '#type' => 'checkbox',
      '#default_value' => $this->configuration['grant_delete'],
    ];

    $priority = is_null($this->configuration['acl_priority']) ? 0 : $this->configuration['acl_priority'];
    $form['acl_priority'] = [
      '#title' => $this->t('Priority'),
      '#description' => $this->t('Set the acl priority'),
      '#type' => 'number',
      '#default_value' => $priority,
    ];

    $form['old_entity'] = [
      '#type' => 'hidden',
      '#default_value' => $this->configuration['licensed_entity'],
    ];

    $form['acl_id'] = [
      '#type' => 'hidden',
      '#default_value' => $this->configuration['acl_id'],
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateConfigurationForm(array &$form, FormStateInterface $form_state) {
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) {
    $values = $form_state->getValue($form['#parents']);

    $aclId = $values['acl_id'];
    if ($aclId == NULL) {
      $aclId = acl_create_acl('commerce_license_access_control');
    }
    $this->configuration['acl_id'] = $aclId;

    acl_node_remove_acl($values['old_entity'], $aclId);
    acl_node_add_acl($values['licensed_entity'], $aclId, $values['grant_view'], $values['grant_update'], $values['grant_delete'], $values['acl_priority']);

    $this->configuration['licensed_entity'] = $values['licensed_entity'];
    $this->configuration['grant_view'] = $values['grant_view'];
    $this->configuration['grant_update'] = $values['grant_update'];
    $this->configuration['grant_delete'] = $values['grant_delete'];
    $this->configuration['acl_priority'] = $values['acl_priority'];
  }

  /**
   * {@inheritdoc}
   */
  public function buildFieldDefinitions() {
    $fields = parent::buildFieldDefinitions();

    $fields['licensed_entity'] = BundleFieldDefinition::create('entity_reference')
      ->setLabel(t('Entity'))
      ->setDescription(t('The node this product grants access on.'))
      ->setCardinality(1)
      ->setRequired(TRUE)
      ->setSetting('target_type', 'node');

    $fields['acl_id'] = BundleFieldDefinition::create('string')
      ->setLabel(t('ACL ID'))
      ->setDescription(t('The id of the ACL that this license grants access to.'))
      ->setCardinality(1)
      ->setRequired(TRUE);

    return $fields;
  }

}
